// Copyright (c) 2019 Marshall A. Greenblatt. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//    * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above
// copyright notice, this list of conditions and the following disclaimer
// in the documentation and/or other materials provided with the
// distribution.
//    * Neither the name of Google Inc. nor the name Chromium Embedded
// Framework nor the names of its contributors may be used to endorse
// or promote products derived from this software without specific prior
// written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// ---------------------------------------------------------------------------
//
// This file was generated by the CEF translator tool and should not edited
// by hand. See the translator.README.txt file in the tools directory for
// more information.
//
// $hash=0385a38b6761c5dec07bb89a95a007ad3c11bea6$
//

#ifndef CEF_INCLUDE_CAPI_CEF_AUDIO_HANDLER_CAPI_H_
#define CEF_INCLUDE_CAPI_CEF_AUDIO_HANDLER_CAPI_H_
#pragma once

#include "include/capi/cef_base_capi.h"
#include "include/capi/cef_browser_capi.h"

#ifdef __cplusplus
extern "C" {
#endif

///
// Implement this structure to handle audio events All functions will be called
// on the UI thread
///
typedef struct _cef_audio_handler_t {
  ///
  // Base structure.
  ///
  cef_base_ref_counted_t base;

  ///
  // Called when the stream identified by |audio_stream_id| has started.
  // |audio_stream_id| will uniquely identify the stream across all future
  // cef_audio_handler_t callbacks. OnAudioSteamStopped will always be called
  // after OnAudioStreamStarted; both functions may be called multiple times for
  // the same stream. |channels| is the number of channels, |channel_layout| is
  // the layout of the channels and |sample_rate| is the stream sample rate.
  // |frames_per_buffer| is the maximum number of frames that will occur in the
  // PCM packet passed to OnAudioStreamPacket.
  ///
  void(CEF_CALLBACK* on_audio_stream_started)(
      struct _cef_audio_handler_t* self,
      struct _cef_browser_t* browser,
      int audio_stream_id,
      int channels,
      cef_channel_layout_t channel_layout,
      int sample_rate,
      int frames_per_buffer);

  ///
  // Called when a PCM packet is received for the stream identified by
  // |audio_stream_id|. |data| is an array representing the raw PCM data as a
  // floating point type, i.e. 4-byte value(s). |frames| is the number of frames
  // in the PCM packet. |pts| is the presentation timestamp (in milliseconds
  // since the Unix Epoch) and represents the time at which the decompressed
  // packet should be presented to the user. Based on |frames| and the
  // |channel_layout| value passed to OnAudioStreamStarted you can calculate the
  // size of the |data| array in bytes.
  ///
  void(CEF_CALLBACK* on_audio_stream_packet)(struct _cef_audio_handler_t* self,
                                             struct _cef_browser_t* browser,
                                             int audio_stream_id,
                                             const float** data,
                                             int frames,
                                             int64 pts);

  ///
  // Called when the stream identified by |audio_stream_id| has stopped.
  // OnAudioSteamStopped will always be called after OnAudioStreamStarted; both
  // functions may be called multiple times for the same stream.
  ///
  void(CEF_CALLBACK* on_audio_stream_stopped)(struct _cef_audio_handler_t* self,
                                              struct _cef_browser_t* browser,
                                              int audio_stream_id);
} cef_audio_handler_t;

#ifdef __cplusplus
}
#endif

#endif  // CEF_INCLUDE_CAPI_CEF_AUDIO_HANDLER_CAPI_H_
